/*
 * libkarma/errors.c
 *
 * Copyright (c) Enrique Vidal 2004
 *           (c) Frank Zschockelt <libkarma@freakysoft.de> 2004
 *
 * You may distribute and modify this program under the terms of 
 * the GNU GPL, version 2 or later.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "errors.h"

const char *lkerrorList[MAXLKERRORS+1] = {
  /* E_NOERR      0 */ "",
  /* E_HOST       1 */ "** Libkarma error: gethostbyname() failed",
  /* E_SOCKET     2 */ "** Libkarma error: socket() failed",
  /* E_SENDTO     3 */ "** Libkarma error: sendto() failed",
  /* E_CONNECT    4 */ "** Libkarma error: connect() failed",
  /* E_RECVFROM   5 */ "** Libkarma error: recvfrom() failed",
  /* E_SELECT     6 */ "** Libkarma error: select() failed",
  /* E_OPEN       7 */ "** Libkarma error: open() failed",
  /* E_ICONVO     8 */  "* Libkarma warning: iconv_open() failed",
  /* E_ICONV      9 */  "* Libkarma warning: iconv() failed",
  /* E_ICONVC    10 */ "** Libkarma error: iconv_close() failed",
  /* E_BADHEADER 11 */ "** Libkarma error: broken Rio header",
  /* E_BADIDENT  12 */ "** Libkarma error: wrong protocol identifier",
  /* E_FAILEDREQ 13 */  "* Libkarma warning: failed request",
  /* E_BADPROP   14 */ "** Libkarma error: property parsing failed",
  /* E_BADID     15 */ "** Libkarma error: given id not found",
  /* E_NOSSDP    16 */  "* Libkarma warning: no ssdp answer",
  /* E_SSDPPARSE 17 */  "* Libkarma warning: ssdp parsing failed",
  /* E_MKDIR     18 */ "** Libkarma error: mkdir() failed",
  /* E_UTIME     19 */ "** Libkarma error: utime() failed",
  /* E_NODIR     20 */ "** Libkarma error: Unexisting directory",
  /* E_NOPROP    21 */  "* Libkarma warning: Missing fid properties",
  /* E_READ      22 */  "* Libkarma warning: Read error",
  /* E_WRITE     23 */ "** Libkarma error: Write error",
  /* E_DELETE    24 */ "** Libkarma error: Delete error",
  /* E_UNIMPLEM  25 */  "* Libkarma warning: Unimplemented call"
};

#define libkarmaErrorString lkerrorList[libkarmaError]
#define previousErrorString lkerrorList[previousError]

/* libkarmaErrorString is assumedly valid only if libkarmaErrorFlag == 1 */
static int  libkarmaError = 0;
static int  previousError = 0;
static int  autoPrintErrors = 1;
static error_handler * callback = NULL;

/* ---------------------------- Error Setting --------------------------- */
int lk_errors_set(const int err) {
  lk_errors_number();
  if ((err < 0) || (err >= MAXLKERRORS)) return -1;
  libkarmaError = err;
  if(callback){
	callback();
  }
  if (autoPrintErrors) 
    fprintf(stderr, "%s\n", libkarmaErrorString);
  return 0;
}

/* -------------------------- Error checking -------------------------- */
void lk_errors_autoprint(const int autoPrint) {
  autoPrintErrors = autoPrint;
}

void lk_errors_setHandler(error_handler * handler)
{
	callback=handler;
}

int lk_errors_number() {
  /* checks current error number and backs-up and resets it */
  previousError = libkarmaError;
  libkarmaError = 0;
  return previousError;
}

int lk_errors_number_noreset() {
  return libkarmaError;
}

int lk_errors_p(const char *s1, const char *s2) {
  if (lk_errors_number()) 
    fprintf(stderr, "%s%s%s\n", s1, previousErrorString, s2); 
  return previousError;
}

char * lk_errors_numberstr(int lkerrnum) {
  if ((lkerrnum < 0) || (lkerrnum >= MAXLKERRORS)) return NULL;
  return (char * )lkerrorList[lkerrnum];
}
