/*
 * libkarma/errors.h
 *
 * Copyright (c) Enrique Vidal 2004
 *           (c) Frank Zschockelt <libkarma@freakysoft.de> 2004
 *
 * You may distribute and modify this program under the terms of 
 * the GNU GPL, version 2 or later.
 *
 */
#ifndef _LKERRORS_H
#define _LKERRORS_H

#ifdef __cplusplus
extern "C" {
#endif

typedef void *error_handler(void);

int  lk_errors_set(const int err);                       /* Sets error number */
void lk_errors_autoprint(const int autoPrint); /*Sets errors aoutoprint on/off*/
void lk_errors_setHandler(error_handler handler); /*Sets an error handl. func.*/
int  lk_errors_number(void);             /* checks error number and resets it */
int  lk_errors_number_noreset(void); /* checks error number without resetting */
int  lk_errors_p(const char *s1, const char *s2); /*prints s1+errStr+s2&resets*/
char * lk_errors_numberstr(int lkerrnum);  /* returns err string for lkerrnum */
                                                                /* - no reset */

#define E_NOERR      0 /* Success */
#define E_HOST       1 /* ** Libkarma error: gethostbyname() failed */ /* Sys */
#define E_SOCKET     2 /* ** Libkarma error: socket() failed */        /* Sys */
#define E_SENDTO     3 /* ** Libkarma error: sendto() failed */        /* Sys */
#define E_CONNECT    4 /* ** Libkarma error: connect() failed */       /* Sys */
#define E_RECVFROM   5 /* ** Libkarma error: recvfrom() failed */      /* Sys */
#define E_SELECT     6 /* ** Libkarma error: select() failed */        /* Sys */
#define E_OPEN       7 /* ** Libkarma error: open() failed */          /* Sys */
#define E_ICONVO     8 /*  * Libkarma warning: iconv_open() failed */  /* Sys */
#define E_ICONV      9 /*  * Libkarma warning: iconv() failed */       /* Sys */
#define E_ICONVC    10 /* ** Libkarma error: iconv_close() failed */   /* Sys */

#define E_BADHEADER 11 /* ** Libkarma error: broken Rio header */
#define E_BADIDENT  12 /* ** Libkarma error: wrong protocol identifier */
#define E_FAILEDREQ 13 /*  * Libkarma warning: failed request */

#define E_BADPROP   14 /* ** Libkarma error: property parsing failed */
#define E_BADID     15 /* ** Libkarma error: given id not found */

#define E_NOSSDP    16 /*  * Libkarma warning: no ssdp answer */
#define E_SSDPPARSE 17 /*  * Libkarma warning: ssdp parsing failed */

#define E_MKDIR     18 /* ** Libkarma error: mkdir() failed */         /* Sys */
#define E_UTIME     19 /* ** Libkarma error: utime() failed */         /* Sys */

#define E_NODIR     20 /* ** Libkarma error: Unexisting directory */   /* USB */
#define E_NOPROP    21 /*  * Libkarma warning: Missing prop file */    /* USB */
#define E_READ      22 /* ** Libkarma error: Read error */             /* USB */
#define E_WRITE     23 /* ** Libkarma error: Write error */            /* USB */
#define E_DELETE    24 /*  * Libkarma warning: Delete error */         /* USB */
#define E_UNIMPLEM  25 /*  * Libkarma warning: Unimplemented call */   /* USB */

#define MAXLKERRORS 25

#ifdef __cplusplus
}
#endif  /* C++ */

#endif
