/*
 * libkarma/karma.c
 *
 * Copyright (c) Frank Zschockelt <libkarma@freakysoft.de> 2004-2005
 * Copyright (c) Enrique Vidal <evidal@iti.upv.es> 2006
 *
 * You may distribute and modify this program under the terms of 
 * the GNU GPL, version 2 or later.
 *
 */

#include "karma.h"
#include "karmaLan.h"
#include "karmaUsb.h"
#include "errors.h"
/* #include "properties.h" */

#define UNIMPL lk_errors_set(E_UNIMPLEM)
int USB=0;

/* -------------------------------------------------------------------------- */
/* Basic Protocol Functions */
int lk_karma_connect(char *ipHostOrPath){
  USB = (ipHostOrPath[0]=='/');
  return USB ? lk_karmaUsb_connect(ipHostOrPath)
             : lk_karmaLan_connect(ipHostOrPath);
}

int32_t lk_karma_hangup(int rio){
  return USB ? lk_karmaUsb_hangup(rio)
             : lk_karmaLan_hangup(rio); 
}

int lk_karma_send_request(int rio,
			  uint32_t identifier, char *payload, int plen){
  return USB ? UNIMPL : lk_karmaLan_send_request(rio, identifier,payload,plen);
}

int lk_karma_get_protocol_version(int rio, uint32_t *major_version,
				  uint32_t *minor_version){
  return USB ?
  UNIMPL : lk_karmaLan_get_protocol_version(rio, major_version, minor_version); 
}

int32_t lk_karma_get_authentication_salt(int rio, char **salt){
  return USB ? UNIMPL : lk_karmaLan_get_authentication_salt(rio, salt);
}

uint32_t lk_karma_authenticate(int rio, char *pass){
  return USB ? lk_karmaUsb_authenticate(rio, pass)
             : lk_karmaLan_authenticate(rio, pass);
}

int lk_karma_get_device_details(int rio, char **name, char **version, 
				uint32_t *storagedevices){
  return USB ?
  UNIMPL : lk_karmaLan_get_device_details(rio, name, version, storagedevices);
}

int lk_karma_get_storage_details(int rio, uint32_t storage_id, 
				 uint32_t *n_files, uint64_t *s_size, 
				 uint64_t *f_space, uint32_t *highest_file_id){
  return USB ? lk_karmaUsb_get_storage_details(rio, storage_id, n_files, s_size,
					       f_space, highest_file_id)
             : lk_karmaLan_get_storage_details(rio, storage_id, n_files, s_size,
					       f_space, highest_file_id); 
}

int lk_karma_get_device_settings(int rio, char **properties){
  return USB ? lk_karmaUsb_get_device_settings(rio, properties)
             : lk_karmaLan_get_device_settings(rio, properties); 
}

int32_t lk_karma_update_device_settings(int rio, char *properties){
  return USB ? UNIMPL : lk_karmaLan_update_device_settings(rio, properties); 
}

int32_t lk_karma_request_io_lock(int rio, uint32_t type){
  return USB ? lk_karmaUsb_request_io_lock(rio, type)
             : lk_karmaLan_request_io_lock(rio, type); 
}

int32_t lk_karma_release_io_lock(int rio){
  return USB ? lk_karmaUsb_release_io_lock(rio)
             : lk_karmaLan_release_io_lock(rio); 
}

/* Not yet supported by the Rio Karma.*/
int32_t lk_karma_prepare(int rio, uint64_t size, 
			 uint32_t *file_id, uint32_t storage_id){
  return USB ? UNIMPL : lk_karmaLan_prepare(rio, size, file_id, storage_id); 
}

int32_t lk_karma_get_all_file_details(int rio, char **properties){
  return USB ? UNIMPL : lk_karmaLan_get_all_file_details(rio, properties); 
}

int32_t lk_karma_get_file_details(int rio, uint32_t file_id, char **properties){
  return USB ? UNIMPL : lk_karmaLan_get_file_details(rio, file_id, properties); 
}

int32_t lk_karma_update_file_details(int rio, 
				     uint32_t file_id, char *properties){
  return USB ? lk_karmaUsb_update_file_details(rio, file_id, properties)
             : lk_karmaLan_update_file_details(rio, file_id, properties); 
}

int32_t lk_karma_read_file_chunk(int rio, uint64_t offset, 
				 uint64_t size, uint32_t file_id, 
				 char **data, uint64_t *retsize){
  return USB ?
    lk_karmaUsb_read_file_chunk(rio, offset, size, file_id, data, retsize) : 
    lk_karmaLan_read_file_chunk(rio, offset, size, file_id, data, retsize); 
}

int32_t lk_karma_write_file_chunk(int rio, uint64_t offset, uint64_t size, 
			 uint32_t file_id, uint32_t storage_id, char *data){
  return USB ?
    lk_karmaUsb_write_file_chunk(rio, offset, size, file_id, storage_id, data) :
    lk_karmaLan_write_file_chunk(rio, offset, size, file_id, storage_id, data); 
}

int32_t lk_karma_delete_file(int rio, uint32_t file_id){
  return USB ? lk_karmaUsb_delete_file(rio, file_id)
             : lk_karmaLan_delete_file(rio, file_id); 
}

/* Now, that command would be scary ;)
   (It's not supported by the rio karma ;)) */
int32_t lk_karma_format_storage(int rio, uint32_t storage_id){
  return USB ? UNIMPL : lk_karmaLan_format_storage(rio, storage_id); 
}

/* Not yet supported by the Rio Karma */
int32_t lk_karma_device_operation(int rio, uint64_t size, 
				  char *data, char **newdata){
  return USB ? UNIMPL : lk_karmaLan_device_operation(rio, size, data, newdata); 
}

/* Advanced Protocol Functions */
void lk_karma_load_database(int rio){
  USB ? lk_karmaUsb_load_database(rio)
      : lk_karmaLan_load_database(rio); 
}

void lk_karma_update_database(int rio){
  if (!USB) lk_karmaLan_update_database(rio); 
}
