/*
 * libkarma/karma.c
 *
 * Copyright (c) Frank Zschockelt <libkarma@freakysoft.de> 2004-2005
 * Copyright (c) Enrique Vidal <evidal@iti.upv.es> 2006
 *
 * You may distribute and modify this program under the terms of 
 * the GNU GPL, version 2 or later.
 */

#define _XOPEN_SOURCE 500  /* Needed by pread/pwrite */
#include <dirent.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <sys/stat.h>
#include <sys/statfs.h>
#include <sys/vfs.h>    /* or <sys/statfs.h> */
#include <sys/types.h>
#include <unistd.h>

#include "karmaUsb.h"
#include "errors.h"
#include "properties.h"
#include "md5.h"
#include "util.h"

#ifndef MSG_FIN
#define MSG_FIN 0
#endif

char *usbMountPoint=NULL;

/* -------------------------------------------------------------------------- */
/* Auxiliary Function */
/* -------------------------------------------------------------------------- */
static char *fidToPath(int rio, uint32_t file_id){
  int i, len;
  char *path=NULL;

  len = strlen(usbMountPoint) + 17;  path = malloc(len);
  sprintf(path, "%sfids0/_%.8x", usbMountPoint, file_id); len = strlen(path);
  for(i=len; i>=len-3; i--) path[i+1] = path[i];  path[len-3] = '/';
  return path;
}

/* -------------------------------------------------------------------------- */
/* Basic Protocol Functions */
/* -------------------------------------------------------------------------- */
int lk_karmaUsb_connect(char *path){
/* ******************** Implemented OMFS mount here ?? ********************** */
/* The path must be absolute to be accesible in lkarmafs;  e.g.: /mnt/karma2/ */
  int len = strlen(path);

  usbMountPoint = malloc(len + 2); strcpy(usbMountPoint, path);
  if(path[len-1] != '/') strcat(usbMountPoint, "/"); 
  return 0;
}

/* -------------------------------------------------------------------------- */
int32_t lk_karmaUsb_hangup(int rio){
/* ********************* Implemented OMFS umount here ?? ******************** */
  return 0;
}

/* -------------------------------------------------------------------------- */
uint32_t lk_karmaUsb_authenticate(int rio, char *pass){
/* *************** Implement some permission checking here ?? *************** */
  return 0;
}

/* -------------------------------------------------------------------------- */
int lk_karmaUsb_get_storage_details(int rio, 
                                    uint32_t storage_id, uint32_t *n_files, 
                                    uint64_t *s_size,    uint64_t *f_space, 
                                    uint32_t *highest_file_id){
  int aux;
  struct statfs *buf;

  buf = malloc(sizeof(struct statfs));
  aux = statfs(usbMountPoint, buf); 
/*perror("+++statfs"); */
  if(aux == -1) {lk_errors_set(E_FAILEDREQ); return -1;}

  *n_files = lk_htorll(buf->f_files - buf->f_ffree); 
  *s_size  = lk_htorll(1024*buf->f_bfree);
  *f_space = lk_htorll(1024*buf->f_bavail);
  *highest_file_id = lk_htorll(buf->f_files);

  free(buf);

  return 0;

/* copied from my test karma */
/*   static uint32_t files = 0,        hfid  = 372613120; */
/*   static uint64_t size  = 19453096, space = 17141096; // 1024-byte blocks */
/*   *n_files = lk_htorl(files);      *highest_file_id = lk_htorl(hfid); */
/*   *s_size  = lk_htorll(1024*size); *f_space = lk_htorll(1024*space); */
}

/* -------------------------------------------------------------------------- */
int lk_karmaUsb_get_device_settings(int rio, char **properties){
/* ****************** To be implemented by parsing smalldb ****************** */
/* "device_generation=1142195096\n" */ /* copied from my test karma 12-Mar-06 */
/* "device_generation=4294967295\n" */ /* max unsigned int  */
  const char *settings = \
  "capabilities="
    ";read_content;utf8;rewrite_metadata;file_replace" \
    ";playlists;root_playlist;played_statistics;\n" \
  "codecs=;mp3;wma;vorbis;flac;wave;\n" \
  "device_generation=4294967295\n" \
  "device_id=0x5210\n" \
  "firmware_version=1.68\n" \
  "hwrev=0x4\n" \
  "manufacturer=Rio Audio\n" \
  "max_playlist_entries=15000\n" \
  "name=Rio Karma\n" \
  "recovery=0\n" \
  "rns_prefix=KRM\n" \
  "rns_version=805016800\n" \
  "serial=0046412\n" \
  "storage_name=Hard disk\n";

  *properties = strdup(settings);
  return 0;
}

/* -------------------------------------------------------------------------- */
int32_t lk_karmaUsb_request_io_lock(int rio, uint32_t type){
/* ****************************** UNNEEDED ??? ****************************** */
  return 0;
}

int32_t lk_karmaUsb_release_io_lock(int rio){
/* ****************************** UNNEEDED ??? ****************************** */
  return 0;
}

/* -------------------------------------------------------------------------- */
/* Esential Functions */
/* -------------------------------------------------------------------------- */
static int32_t read_properties(char **properties){
  DIR *dp0, *dp1, *dp;
  struct dirent *de0, *de1, *de;
  int oneFile;
  char path1[1024]="", path[1024]="";
  int numFiles=0, i=0, j=0, got=0;
  uint32_t fid;

  dp0 = opendir(usbMountPoint);
  if(dp0 == NULL) {lk_errors_set(E_NODIR); return -1;}
  while((de0 = readdir(dp0)) != NULL) if(strncmp(de0->d_name, "fids", 4)==0) {
    strcpy(path1, usbMountPoint);
    strcat(path1, de0->d_name); strcat(path1, "/"); 
    dp1 = opendir(path1);
    if(dp1 == NULL) {lk_errors_set(E_NODIR); return -1;}
    while((de1 = readdir(dp1)) != NULL) if(strncmp(de1->d_name, "_00", 3)==0) {
      strcpy(path, path1); strcat(path, de1->d_name); strcat(path, "/"); 
      dp = opendir(path); 
      if(dp == NULL) {lk_errors_set(E_NODIR); return -1;}
      while((de = readdir(dp)) != NULL)
      if((strcmp(de->d_name, ".")) && (strcmp(de->d_name, ".."))) {
        char realFile[1024], fidLine[32], *aux;
        if(de->d_name[strlen(de->d_name)-1] == '0') {
          aux = strcpy(realFile, path);
          aux = strcat(realFile, de->d_name);
/*        fprintf(stderr, "--> %s\n", realFile); */
          realFile[strlen(realFile)-1] = '1';
          oneFile = open(realFile, O_RDONLY);
          if(oneFile == -1) {
/*          fprintf(stderr,"\nWarning: wrong file (%s)\n", realFile); */
	    lk_errors_set(E_NOPROP);
            break;
          }
	  if(j-i < 1024+32) {
	    *properties = realloc(*properties, (i+2048));  j = i + 2048;
	  }
	  strncpy(fidLine, rindex(path, '_')+1, 16);
	  fidLine[strlen(fidLine)-1] = '\0';
	  strncat(fidLine, de->d_name, 16-strlen(fidLine)); 
	  fid = strtol(fidLine, NULL, 16);
/*        fprintf(stderr,"%s %d\n", fidLine, fid); */
	  sprintf(fidLine, "fid=%d\n", fid);
	  aux = strcpy(*properties + i, fidLine);  i += strlen(fidLine);
	  got = read(oneFile, *properties + i, 1024);
	  if(got == -1 || got > 1022) {lk_errors_set(E_FAILEDREQ);}
/*        fprintf(stderr, "\n%s\n-------\n", *properties+i);  */
	  i += got;  *(*properties + i) = '\n';  i++;
          close(oneFile);
	  numFiles++;
/*        fprintf(stderr, "\rReading: %3d files. Please wait... ", numFiles); */
        }
      } closedir(dp);
    } closedir(dp1);
  } closedir(dp0);
  if(*properties) *(*properties + i) = '\0';
  else {lk_errors_set(E_FAILEDREQ); return -1;}
/*fprintf(stderr, "\n"); */
  return 0;
}
/* .......................................................................... */
int32_t lk_karmaUsb_get_all_file_details(int rio, char **properties){
  return read_properties(properties);
}

/* -------------------------------------------------------------------------- */
int32_t lk_karmaUsb_read_file_chunk(int rio,       uint64_t offset, 
                                    uint64_t size, uint32_t file_id, 
                                    char **data,   uint64_t *retsize){
  int  fd;
  int  res;
  char *path=NULL;

  path = fidToPath(rio, file_id);  fd = open(path, O_RDONLY);  free(path);
/*if(offset==0) fprintf(stderr, "--> %s %x %d\n", path, file_id, file_id); */
/*fprintf(stderr, "---> %s\n", strerror(errno)); */
  if(fd == -1) {close(fd); lk_errors_set(E_READ); return -1;}

  *data=(char *)malloc((size_t)size);
  res = pread(fd, *data, (size_t)size, (size_t)offset);
/*fprintf(stderr, "--> %d\n", res); */
  if(res>=0) *retsize = res;
  else       {close(fd); lk_errors_set(E_READ); return -1;}

  close(fd);
  return 0;
}

/* -------------------------------------------------------------------------- */
int32_t lk_karmaUsb_write_file_chunk(int rio,       uint64_t offset, 
                                     uint64_t size, uint32_t file_id, 
                                     uint32_t storage_id, char *data){
#define PERMS S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH
  int  fd;
  int  res;
  char *path=NULL;

  path = fidToPath(rio, file_id);
/*if(offset==0) fprintf(stderr, "--> %s %x %d\n", path, file_id, file_id); */
  if((fd = open(path, O_WRONLY|O_CREAT, PERMS)) == EEXIST)
      fd = open(path, O_WRONLY|O_TRUNC, PERMS);
  free(path);
/*fprintf(stderr, "---> %s\n", strerror(errno)); */
  if(fd == -1) {close(fd); lk_errors_set(E_WRITE); return -1;}

  res = pwrite(fd, data, (size_t)size, (size_t)offset);
/*fprintf(stderr, "--> %d\n", res); */
  if(res<0) {close(fd); lk_errors_set(E_WRITE); return -1;}

  close(fd);
  return 0;
}

/* -------------------------------------------------------------------------- */
int32_t lk_karmaUsb_update_file_details(int rio, 
                                        uint32_t file_id, char *properties){
/* Modify (or create) the *1 file of file_id with properties */
/* ************************* UPDATE smalldb TOO ?? ************************** */
#define PERMS S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH
  int  fd;
  int  len, res;
  char *path=NULL;

  if(!properties) {lk_errors_set(E_FAILEDREQ); return -1;}

  path = fidToPath(rio, file_id);  path[strlen(path)-1] = '1';
/*fprintf(stderr, "--> %s %x %d\n", path, file_id, file_id); */

  unlink(path); fd = open(path, O_WRONLY|O_CREAT, PERMS);
  free(path);
/*fprintf(stderr, "---> %s\n", strerror(errno)); */
  if(fd == -1) {close(fd); lk_errors_set(E_WRITE); return -1;}

  len = strlen(properties);  res = write(fd, properties, len);
/*fprintf(stderr,"--> len=%d %d\n", len, res); */
  if(res == -1) {close(fd); lk_errors_set(E_WRITE); return -1;}

  close(fd);
  return 0;
}

/* -------------------------------------------------------------------------- */
int32_t lk_karmaUsb_delete_file(int rio, uint32_t file_id){

  int  ret=0;
  char *path=NULL;

  path = fidToPath(rio, file_id); 
/*fprintf(stderr, "--> %s %x %d\n", path, file_id, file_id); */
  if(unlink(path)) ret = -1;
  
  path[strlen(path)-1] = '1';
/*fprintf(stderr, "--> %s\n", path); */
  if(unlink(path)) ret = -1;

  if(ret == -1) lk_errors_set(E_DELETE);
  free(path);
  return ret;
}

/* -------------------------------------------------------------------------- */
/* Advanced Protocol Functions */
/* -------------------------------------------------------------------------- */
void lk_karmaUsb_load_database(int rio){
  int count=0, aux=0; char *settings=NULL, *properties=NULL;

  aux = lk_karmaUsb_get_device_settings(rio, &settings);
  if(lk_properties_cache_obsolete(settings)){ /* DB was changed */
    free(settings); settings=NULL;
    aux = read_properties(&properties);
    if(properties) count = lk_properties_import(properties);
    else {lk_errors_set(E_FAILEDREQ); return;}
/*  fprintf(stderr, "%d Files\n", count); */
  }
  else lk_properties_load();
  free(settings); free(properties);
}

/* -------------------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */
/* THE LAN VERSIONS USED ONLY INTERNALLY IN karmaLan.c 
int lk_karmaUsb_get_protocol_version(int rio, uint32_t *major_version,
                                             uint32_t *minor_version){return 0;}
int lk_karmaUsb_send_request(int rio, uint32_t identifier, 
			                    char * payload, int plen){return 0;}
int32_t lk_karmaUsb_get_authentication_salt(int rio, char **salt)    {return 0;}
*/
/* -------------------------------------------------------------------------- */
/* UNUSED
int     lk_karmaUsb_get_device_details(int rio, char **name, char **version, 
				            uint32_t *storagedevices){return 0;}
int32_t lk_karmaUsb_get_file_details(int rio, uint32_t file_id, 
        			                   char **properties){return 0;}
 
int32_t lk_karmaUsb_update_device_settings(int rio, char *properties){return 0;}
int32_t lk_karmaUsb_prepare(int rio, uint64_t size, 
                              uint32_t *file_id, uint32_t storage_id){return 0;}
int32_t lk_karmaUsb_format_storage(int rio, uint32_t storage_id)     {return 0;}
int32_t lk_karmaUsb_device_operation(int rio, uint64_t size, 
				     char *data, char **newdata)     {return 0;}
void lk_karmaUsb_update_database(int rio){}
*/
/* -------------------------------------------------------------------------- */
