/*
 * libkarma/properties.c
 * 
 * Copyright (c) Frank Zschockelt <libkarma@freakysoft.de> 2004-2006
 * 
 * You may distribute and modify this program under the terms of 
 * the GNU GPL, version 2 or later.
 * 
 */
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <utime.h>

#include "properties.h"
#include "util.h"
#include "errors.h"

#define HASHSIZE 16
#define TABLECHUNK 4096
#define FIRSTFID 17 /* 272/16 */

struct klist{
	HASH ** fidTable;
	uint32_t ftsize;
	uint32_t count;
	uint32_t next_fid;
};

struct klist props = {NULL, 0, 0, 17};

void lk_properties_init(void)
{
	uint32_t i;
	props.count=0;
	props.next_fid=17;
	props.ftsize=TABLECHUNK;
	props.fidTable=malloc(props.ftsize*sizeof(HASH *));
	for(i=0;i<props.ftsize;i++) 
		props.fidTable[i]=NULL;
}

void lk_properties_destroy(void)
{
	uint32_t i=0;
	
	while(props.count > 0) {
		if(props.fidTable[i]){
			hash_destroy(&props.fidTable[i]);
			props.count--;
		}
		i++;
	}
	free(props.fidTable);
	props.fidTable=NULL;
	props.count=0;
	props.next_fid=17;
}

uint32_t lk_properties_import(char * properties)
{
	uint32_t count=0;
	char *key, *value;
	char *ikey, *ivalue;
	HASH * item;
	uint32_t fid,i;
	char *n_tok, *last;
	
	last=&(properties[strlen(properties)])+1; 
	/* otherwise we can't see whether n_tok is outside of properties */

	item=hash_create(HASHSIZE);
	hash_insert(item, strdup("fid"), strdup("0"));
	key=strtok_r(properties, "=", &n_tok);
	if(key==NULL){
		lk_errors_set(E_BADPROP);
		return -1;
	}				
/*      fprintf(stderr, "n_tok = --%c--\n", *n_tok); */
	if(*n_tok == '\n') {value=""; n_tok++;}
	else value=strtok_r(NULL, "\n", &n_tok);

	while(value != NULL){
		ikey=strdup(key);
		ivalue=strdup(value);
		hash_insert(item, ikey, ivalue);
/*              fprintf(stderr, "hash_insert -%s--%s-\n", ikey, ivalue); */

		if((n_tok[0] == '\n')||(n_tok[0]=='\0')){
			++n_tok;
			
			fid=(uint32_t)(atol(lk_properties_get_property_hash(item, "fid"))/16);
			while(fid >= props.ftsize){
				i=props.ftsize+TABLECHUNK;
				props.fidTable=realloc(props.fidTable, i*sizeof(HASH *));
				for( ;props.ftsize<i;props.ftsize++)
					props.fidTable[props.ftsize]=NULL;
			}
			if(props.fidTable[fid])
				hash_destroy(&props.fidTable[fid]);
			else
				++props.count;
			
			props.fidTable[fid]=item;
			
			item=hash_create(HASHSIZE);
			++count;
		}
		if(n_tok==last)
			key=value=NULL;
		else{
			key=strtok_r(NULL, "=", &n_tok);
/*                      fprintf(stderr, "n_tok = --%c--\n", *n_tok); */
                        if(*n_tok == '\n') {value=""; n_tok++;}
			else value=strtok_r(NULL, "\n", &n_tok);
		}
	}
	hash_destroy(&item);
	return count;
}

char * lk_properties_export(uint32_t id)
{
	HASH *table; struct hash_elem *tmp; 
	char *p;
	int len, i;

#define	export_hash(key, data) { \
	len+=strlen(key)+strlen((char *)data)+6; \
	p=realloc(p, len); \
	strcat(p, key); \
	strcat(p, "="); \
	strcat(p, (char*)data); \
	strcat(p, "\n"); \
	}

	len=0;
	p=malloc(1);
	*p=0;
	table=lk_properties_idsearch(id);
	if (table==NULL) {
		lk_errors_set(E_BADID);
		return NULL;
	}
	else {
		for (i=0; i < table->nelem; i++) {
			tmp=table->table[i];
			while(tmp){
				export_hash(tmp->key, tmp->data);
				tmp=tmp->next;
			}
		}
	}
	return p;
}

/* The karma only plays files when the ID is > 256 and ID%16==0 */
uint32_t lk_properties_getnextfid(void)
{
	while(props.fidTable[props.next_fid])
		props.next_fid++;
	return props.next_fid*16;
}

uint32_t lk_properties_new_property(void)
{
	HASH *item;
	static uint32_t new_fid;
	uint32_t i;
	
	item=hash_create(HASHSIZE);
	new_fid=lk_properties_getnextfid();
	lk_properties_set_property_hash(item, "fid", simple_itoa(new_fid));
	
	while(new_fid/16 >= props.ftsize){
		i=props.ftsize+TABLECHUNK;
		props.fidTable=realloc(props.fidTable, i*sizeof(HASH *));
		for( ;props.ftsize<i;props.ftsize++)
			props.fidTable[props.ftsize]=NULL;
	}
	props.fidTable[new_fid/16]=item;
	
	props.next_fid++;
	props.count++;
	return new_fid;
}


char * lk_properties_get_property(uint32_t fid, char * key)
{
	HASH *tmp;

	tmp=lk_properties_idsearch(fid);
	if(tmp==NULL){
		lk_errors_set(E_BADID);
		return NULL;
	}
	else return (char *)hash_find(tmp, key);
}

char * lk_properties_get_property_hash(HASH * p, char * key)
{
	return (char *)hash_find(p, key);
}

int lk_properties_del_property(uint32_t id)
{
	id/=16;
	if(props.fidTable[id] == NULL){
		lk_errors_set(E_BADID);
		return -1;
	}

	hash_destroy(&props.fidTable[id]);
	props.fidTable[id]=NULL;

	props.count--;
	if(id < props.next_fid)
		props.next_fid=id;
	return 0;
}

int lk_properties_set_property(uint32_t id, char * property, char * data)
{
	id/=16;
	if(props.fidTable[id] == NULL){
		lk_errors_set(E_BADID);
		return -1;
	}
	lk_properties_set_property_hash(props.fidTable[id], property, data);
	
	return 0;
}

void lk_properties_set_property_hash(HASH * p, char * property, char * data)
{
	if (data == NULL) return;

	hash_insert(p, strdup(property), strdup(data));
}

HASH * lk_properties_idsearch(uint32_t fid)
{
	return props.fidTable[fid/16];
}

int lk_properties_load(void)
{
	int fd;
	char * buf;
	struct stat info;
	char *fname;

	fname=lk_path_string("fileinfo");

	fd=open(fname,O_RDONLY);
	if(fd==-1) {
		free(fname);
		lk_errors_set(E_OPEN);
		return -1;
	}
	fstat(fd, &info);
	buf=(char *)malloc((int)info.st_size+1);
	read(fd, buf, info.st_size);
	close(fd);
	buf[info.st_size]='\0';
	lk_properties_import(buf);
	free(buf);
	free(fname);
	return 0;
}
/*
   caches are stored in $HOME/.openrio/pearl-$SERIAL.

   The $SERIAL value is the Karma's unique serial number.
   This allows multiple Karmas to sync against the same machine without 
   collisions.

   File Format
   At this time, there is only one file specified within the cache 
   directory, named fileinfo. 
   This is a standard text file containing a dump of the File Info properties, 
   as returned by the Get All File Info message to the device.

   The file's last-modified date is set to the device_generation value provided 
   by the Karma  essentially, the last-modified date of the Karma's internal 
   database. If the cache file's last-modified date and the Karma's 
   device_generation do not match, the cache is considered invalid and is 
   rebuilt using information from the device. 
   */
int lk_properties_save(void)
{
	int fd;
	uint32_t i,j;
	char * fname;
	char * buf;
	struct utimbuf time;

	fname=lk_path_string("fileinfo");
	if(mk_path(fname) == -1){
		free(fname);
		return -1;
	}
	fd=open(fname,O_CREAT|O_WRONLY|O_TRUNC, S_IRUSR|S_IWUSR);
	if(fd==-1) {
		free(fname);
		lk_errors_set(E_OPEN);
		return -1;
	}

	for(i=0,j=16; i<props.count-1; j++) {
		if(props.fidTable[j]){
			buf=lk_properties_export(j*16);
			write(fd, buf, strlen(buf));
			free(buf);
			write(fd, "\n", 1);
			++i;
		}
	}
	close(fd);

	/*time.actime=strtoul(lk_properties_get_property(0, "device_generation"), NULL, 10);*/
	time.actime=(uint32_t)atol(lk_properties_get_property(0, "device_generation"));
	time.modtime=time.actime;
	if(utime(fname, &time)==-1){
		free(fname);
		lk_errors_set(E_UTIME);
		return -1;
	}

	free(fname);
	return 0;
}

void lk_properties_inc_devgeneration(void)
{
	uint32_t dg;

	dg=(uint32_t)atoll(lk_properties_get_property(0, "device_generation"));
	lk_properties_set_property(0, "device_generation", simple_itoa(dg+1));
}


int lk_properties_cache_obsolete(char *device_settings)
{
   	char *fname, *tmp;
	struct stat modified;

	tmp=strdup(device_settings);
	lk_properties_import(tmp);
	free(tmp);

	fname=lk_path_string("fileinfo");
	if(stat(fname, &modified) == -1){
		free(fname);
		return 1;
	}
	free(fname);
	if((uint32_t)atoll(lk_properties_get_property(0, "device_generation")) == 
			(unsigned)modified.st_mtime){
		return 0;
	}else return 1;
}

int lk_properties_db_changed(char *device_settings)
{
	char *old, *tmp;
	int ret;

	tmp=lk_properties_get_property(0, "device_generation");
	if(tmp){
		old=strdup(tmp);
		tmp=strdup(device_settings);
		lk_properties_import(tmp);
		free(tmp);
		ret=strcmp(lk_properties_get_property(0, "device_generation"), old) != 0;
		free(old);
		return ret;
	}else return 1;
}

uint32_t *lk_properties_andOrSearch(int mode, uint32_t *search_in, 
                                              char *key, char *data)
{
/*      Exact/approximate And/Or search support:
	mode == 0 == 0000: Or  approx;          mode == 2 == 0010: Or  exact
	     == 1 == 0001: And approx;               == 3 == 0011: And exact  */

	uint32_t count=1, fid=16;
	char * val;
	int hash, exact=0, and=0;
	int i=0,j=0;
	uint32_t * result=NULL;

	if(mode>1) {exact = 1; and = mode-2;} else {exact = 0; and = mode;}

#define STRMATCH(a,b) \
        exact?((data[0]==0)||(val[0]==0)||(strcmp(val, data)==0)):\
                                          (strstr(val, data)!=NULL)
	if(search_in==NULL){
		if(and)                       /* AND merge with zero elements */
			return NULL;
		hash=hash_string((const unsigned char *)key, HASHSIZE);
		while(count < props.count){
			if(props.fidTable[fid]){
			  ++count;
			  val = (char *)hash_find_hash(props.fidTable[fid], hash, key);
/*                      if((val!=NULL) && STRMATCH(val,data)){ */ /* ->crash! */
			  if(val!=NULL) {
			    if(STRMATCH(val, data)) {
			      ++i;
			      result=realloc(result, i*sizeof(uint32_t));
			      result[i-1]=fid*16;
			    }
			  }
			}
			++fid;
		}
	}
	else if (and) {                                          /* AND merge */
		while(search_in[j] != 0){
			val=lk_properties_get_property(search_in[j], key);
			if(val!=NULL) {
			  if(STRMATCH(val, data)){
			  	++i;
			  	result=realloc(result, i*sizeof(uint32_t));
			  	result[i-1]=search_in[j];
			  }
			}
			++j;
		}
		free(search_in);
	}else{                                                    /* OR merge */
		while(search_in[i] != 0)
			++i;
		++i;
		result=malloc((i) * sizeof(uint32_t));
 		memcpy(result, search_in, (i) * sizeof(uint32_t));
		
		hash=hash_string((const unsigned char *)key, HASHSIZE);
		while(count < props.count){
		  if(props.fidTable[fid]){
			  ++count;
			  val=(char *)hash_find_hash(props.fidTable[fid], hash, key);
			  if(val!=NULL) {
			    if(STRMATCH(val, data)){
			      fid*=16;
			      j=0;
			      while((j<i) &&(result[j] != fid))
			        j++;
			      if(j==i) {
				result=realloc(result, (++i)*sizeof(uint32_t));
				result[i-1]=fid;
			      }
			      fid/=16;
			    }
			  }
		  }
		  ++fid;
		}
		free(search_in);
	}
	if(i){
		++i;
		result=realloc(result, i*sizeof(uint32_t));
		result[i-1]=0;
	}
	return result;
}
